@@ -518,6 +518,7 @@ gst_h264_parse_hrd_parameters (GstH264HRDParams * hrd, NalReader * nr)
   for (sched_sel_idx = 0; sched_sel_idx <= hrd->cpb_cnt_minus1; sched_sel_idx++) {
     READ_UE (nr, hrd->bit_rate_value_minus1[sched_sel_idx]);
     READ_UE (nr, hrd->cpb_size_value_minus1[sched_sel_idx]);
+    READ_UINT8 (nr, hrd->cbr_flag[sched_sel_idx], 1);
   }
 
   READ_UINT8 (nr, hrd->initial_cpb_removal_delay_length_minus1, 5);
@@ -622,7 +623,6 @@ gst_h264_parse_vui_parameters (GstH264SPS * sps, NalReader * nr)
     READ_UE_ALLOWED (nr, vui->max_bits_per_mb_denom, 0, 16);
     READ_UE_ALLOWED (nr, vui->log2_max_mv_length_horizontal, 0, 16);
     READ_UE_ALLOWED (nr, vui->log2_max_mv_length_vertical, 0, 16);
-    READ_UE_ALLOWED (nr, vui->log2_max_mv_length_vertical, 0, 16);
     READ_UE (nr, vui->num_reorder_frames);
     READ_UE (nr, vui->max_dec_frame_buffering);
   }
@@ -747,22 +747,26 @@ slice_parse_ref_pic_list_modification_1 (GstH264SliceHdr * slice,
     NalReader * nr, guint list)
 {
   GstH264RefPicListModification *entries;
-  guint8 *ref_pic_list_modification_flag;
+  guint8 *ref_pic_list_modification_flag, *n_ref_pic_list_modification;
   guint32 modification_of_pic_nums_idc;
   guint i = 0;
 
   if (list == 0) {
     entries = slice->ref_pic_list_modification_l0;
     ref_pic_list_modification_flag = &slice->ref_pic_list_modification_flag_l0;
+    n_ref_pic_list_modification = &slice->n_ref_pic_list_modification_l0;
   } else {
     entries = slice->ref_pic_list_modification_l1;
     ref_pic_list_modification_flag = &slice->ref_pic_list_modification_flag_l1;
+    n_ref_pic_list_modification = &slice->n_ref_pic_list_modification_l1;
   }
 
   READ_UINT8 (nr, *ref_pic_list_modification_flag, 1);
   if (*ref_pic_list_modification_flag) {
-    do {
+    while (1) {
       READ_UE (nr, modification_of_pic_nums_idc);
+      if (modification_of_pic_nums_idc == 3)
+        break;
       if (modification_of_pic_nums_idc == 0 ||
           modification_of_pic_nums_idc == 1) {
         READ_UE_ALLOWED (nr, entries[i].value.abs_diff_pic_num_minus1, 0,
@@ -770,9 +774,10 @@ slice_parse_ref_pic_list_modification_1 (GstH264SliceHdr * slice,
       } else if (modification_of_pic_nums_idc == 2) {
         READ_UE (nr, entries[i].value.long_term_pic_num);
       }
-    } while (modification_of_pic_nums_idc != 3);
+      entries[i++].modification_of_pic_nums_idc = modification_of_pic_nums_idc;
+    }
   }
-
+  *n_ref_pic_list_modification = i;
   return TRUE;
 
 error:
@@ -1050,6 +1055,8 @@ gst_h264_parse_clock_timestamp (GstH264ClockTimestamp * tim,
   if (time_offset_length > 0)
     READ_UINT32 (nr, tim->time_offset, time_offset_length);
 
+  return TRUE;
+
 error:
   GST_WARNING ("error parsing \"Clock timestamp\"");
   return FALSE;
@@ -1091,6 +1098,7 @@ gst_h264_parser_parse_pic_timing (GstH264NalParser * nalparser,
       guint8 num_clock_num_ts;
       guint i;
 
+      tim->pic_struct_present_flag = TRUE;
       READ_UINT8 (nr, tim->pic_struct, 4);
       CHECK_ALLOWED ((gint8) tim->pic_struct, 0, 8);
 
@@ -1907,6 +1915,9 @@ gst_h264_parser_parse_sei (GstH264NalParser * nalparser, GstH264NalUnit * nalu,
 
   nal_reader_init (&nr, nalu->data + nalu->offset + 1, nalu->size - 1);
 
+  /* init */
+  memset (sei, 0, sizeof (*sei));
+
   sei->payloadType = 0;
   do {
     READ_UINT8 (&nr, payload_type_byte, 8);
