@@ -117,7 +117,9 @@ static GstStaticPadTemplate mpegtsmux_sink_factory =
         "video/x-dirac;"
         "video/x-h264,stream-format=(string)byte-stream;"
         "audio/mpeg, "
-        "mpegversion = (int) { 1, 2, 4 };"
+        "mpegversion = (int) { 1, 2 };"
+        "audio/mpeg, "
+        "mpegversion = (int) 4, stream-format = (string) { raw, adts };"
         "audio/x-lpcm, "
         "width = (int) { 16, 20, 24 }, "
         "rate = (int) { 48000, 96000 }, "
@@ -888,7 +890,7 @@ mpegtsmux_collected (GstCollectPads * pads, MpegTsMux * mux)
       return GST_FLOW_ERROR;
     }
 
-    if (mux->force_key_unit_event != NULL) {
+    if (mux->force_key_unit_event != NULL && best->stream->is_video_stream) {
       GstEvent *event;
 
       event = check_pending_key_unit_event (mux->force_key_unit_event,
@@ -920,6 +922,7 @@ mpegtsmux_collected (GstCollectPads * pads, MpegTsMux * mux)
 
           program->last_pmt_ts = -1;
         }
+        tsmux_program_set_pcr_stream (prog, NULL);
       }
     }
 
