@@ -179,12 +179,14 @@ gst_h264_parse_reset_frame (GstH264Parse * h264parse)
   /* done parsing; reset state */
   h264parse->nalu.valid = FALSE;
   h264parse->nalu.offset = 0;
+  h264parse->nalu.sc_offset = 0;
   h264parse->nalu.size = 0;
   h264parse->current_off = 0;
 
   h264parse->picture_start = FALSE;
   h264parse->update_caps = FALSE;
   h264parse->idr_pos = -1;
+  h264parse->sei_pos = -1;
   h264parse->keyframe = FALSE;
   h264parse->frame_start = FALSE;
   gst_adapter_clear (h264parse->frame_out);
@@ -212,6 +214,10 @@ gst_h264_parse_reset (GstH264Parse * h264parse)
   h264parse->last_report = GST_CLOCK_TIME_NONE;
   h264parse->push_codec = FALSE;
 
+  h264parse->dts = GST_CLOCK_TIME_NONE;
+  h264parse->ts_trn_nb = GST_CLOCK_TIME_NONE;
+  h264parse->do_ts = TRUE;
+
   h264parse->pending_key_unit_ts = GST_CLOCK_TIME_NONE;
   h264parse->force_key_unit_event = NULL;
 
@@ -285,6 +291,9 @@ gst_h264_parse_get_string (GstH264Parse * parse, gboolean format, gint code)
 static void
 gst_h264_parse_format_from_caps (GstCaps * caps, guint * format, guint * align)
 {
+  g_return_if_fail (gst_caps_is_fixed (caps));
+
+  GST_DEBUG ("parsing caps: %" GST_PTR_FORMAT, caps);
 
   if (format)
     *format = GST_H264_PARSE_FORMAT_NONE;
@@ -318,19 +327,41 @@ gst_h264_parse_format_from_caps (GstCaps * caps, guint * format, guint * align)
 
 /* check downstream caps to configure format and alignment */
 static void
-gst_h264_parse_negotiate (GstH264Parse * h264parse)
+gst_h264_parse_negotiate (GstH264Parse * h264parse, GstCaps * in_caps)
 {
   GstCaps *caps;
   guint format = GST_H264_PARSE_FORMAT_NONE;
   guint align = GST_H264_PARSE_ALIGN_NONE;
 
+  g_return_if_fail ((in_caps == NULL) || gst_caps_is_fixed (in_caps));
+
   caps = gst_pad_get_allowed_caps (GST_BASE_PARSE_SRC_PAD (h264parse));
   GST_DEBUG_OBJECT (h264parse, "allowed caps: %" GST_PTR_FORMAT, caps);
 
-  gst_h264_parse_format_from_caps (caps, &format, &align);
+  /* concentrate on leading structure, since decodebin2 parser
+   * capsfilter always includes parser template caps */
+  if (caps) {
+    caps = gst_caps_make_writable (caps);
+    gst_caps_truncate (caps);
+    GST_DEBUG_OBJECT (h264parse, "negotiating with caps: %" GST_PTR_FORMAT,
+        caps);
+  }
 
-  if (caps)
+  if (in_caps && caps) {
+    if (gst_caps_can_intersect (in_caps, caps)) {
+      GST_DEBUG_OBJECT (h264parse, "downstream accepts upstream caps");
+      gst_h264_parse_format_from_caps (in_caps, &format, &align);
+      gst_caps_unref (caps);
+      caps = NULL;
+    }
+  }
+
+  if (caps) {
+    /* fixate to avoid ambiguity with lists when parsing */
+    gst_pad_fixate_caps (GST_BASE_PARSE_SRC_PAD (h264parse), caps);
+    gst_h264_parse_format_from_caps (caps, &format, &align);
     gst_caps_unref (caps);
+  }
 
   /* default */
   if (!format)
@@ -474,6 +505,15 @@ gst_h264_parse_process_nal (GstH264Parse * h264parse, GstH264NalUnit * nalu)
               GST_TIME_ARGS (h264parse->ts_trn_nb));
           break;
       }
+      /* mark SEI pos */
+      if (h264parse->sei_pos == -1) {
+        if (h264parse->format == GST_H264_PARSE_FORMAT_AVC)
+          h264parse->sei_pos = gst_adapter_available (h264parse->frame_out);
+        else
+          h264parse->sei_pos = nalu->offset - 4;
+        GST_DEBUG_OBJECT (h264parse, "marking SEI in frame at offset %d",
+            h264parse->sei_pos);
+      }
       break;
 
     case GST_H264_NAL_SLICE:
@@ -517,6 +557,12 @@ gst_h264_parse_process_nal (GstH264Parse * h264parse, GstH264NalUnit * nalu)
         GST_DEBUG_OBJECT (h264parse, "marking IDR in frame at offset %d",
             h264parse->idr_pos);
       }
+      /* if SEI preceeds (faked) IDR, then we have to insert config there */
+      if (h264parse->sei_pos >= 0 && h264parse->idr_pos > h264parse->sei_pos) {
+        h264parse->idr_pos = h264parse->sei_pos;
+        GST_DEBUG_OBJECT (h264parse, "moved IDR mark to SEI position %d",
+            h264parse->idr_pos);
+      }
       break;
     default:
       gst_h264_parser_parse_nal (nalparser, nalu);
@@ -545,10 +591,6 @@ gst_h264_parse_collect_nal (GstH264Parse * h264parse, const guint8 * data,
   GstH264NalUnitType nal_type = nalu->type;
   GstH264NalUnit nnalu;
 
-  if (h264parse->align == GST_H264_PARSE_ALIGN_NAL) {
-    return TRUE;
-  }
-
   GST_DEBUG_OBJECT (h264parse, "parsing collected nal");
   parse_res = gst_h264_parser_identify_nalu (h264parse->nalparser, data,
       nalu->offset + nalu->size, size, &nnalu);
@@ -601,7 +643,7 @@ gst_h264_parse_check_valid_frame (GstBaseParse * parse,
   guint size, current_off = 0;
   gboolean drain;
   GstH264NalParser *nalparser = h264parse->nalparser;
-  GstH264NalUnit nalu = h264parse->nalu;
+  GstH264NalUnit nalu;
 
   /* expect at least 3 bytes startcode == sc, and 2 bytes NALU payload */
   if (G_UNLIKELY (GST_BUFFER_SIZE (buffer) < 5))
@@ -609,7 +651,7 @@ gst_h264_parse_check_valid_frame (GstBaseParse * parse,
 
   /* need to configure aggregation */
   if (G_UNLIKELY (h264parse->format == GST_H264_PARSE_FORMAT_NONE))
-    gst_h264_parse_negotiate (h264parse);
+    gst_h264_parse_negotiate (h264parse, NULL);
 
   /* avoid stale cached parsing state */
   if (!(frame->flags & GST_BASE_PARSE_FRAME_FLAG_PARSING)) {
@@ -624,13 +666,16 @@ gst_h264_parse_check_valid_frame (GstBaseParse * parse,
   size = GST_BUFFER_SIZE (buffer);
 
   drain = FALSE;
+  nalu = h264parse->nalu;
   current_off = h264parse->current_off;
 
+  g_assert (current_off < size);
+
   GST_DEBUG_OBJECT (h264parse, "last parse position %u", current_off);
   while (TRUE) {
     GstH264ParserResult pres;
 
-    if (h264parse->packetized)
+    if (h264parse->packetized_chunked)
       pres =
           gst_h264_parser_identify_nalu_unchecked (nalparser, data, current_off,
           size, &nalu);
@@ -652,7 +697,8 @@ gst_h264_parse_check_valid_frame (GstBaseParse * parse,
           h264parse->nalu = nalu;
 
         /* need 2 bytes of next nal */
-        if (!h264parse->packetized && (nalu.offset + nalu.size + 4 + 2 > size)) {
+        if (!h264parse->packetized_chunked &&
+            (nalu.offset + nalu.size + 4 + 2 > size)) {
           if (GST_BASE_PARSE_DRAINING (parse)) {
             drain = TRUE;
           } else {
@@ -660,6 +706,10 @@ gst_h264_parse_check_valid_frame (GstBaseParse * parse,
             current_off = nalu.sc_offset;
             goto more;
           }
+        } else if (h264parse->packetized_chunked) {
+          /* normal next nal based collection not possible,
+           * _chain will have to tell us whether this was last one for AU */
+          drain = h264parse->packetized_last;
         }
         break;
       case GST_H264_PARSER_BROKEN_LINK:
@@ -668,7 +718,9 @@ gst_h264_parse_check_valid_frame (GstBaseParse * parse,
         current_off = size - 3;
         goto parsing_error;
       case GST_H264_PARSER_NO_NAL:
-        current_off = size - 3;
+        /* don't expect to have found any NAL so far */
+        g_assert (h264parse->nalu.size == 0);
+        current_off = h264parse->nalu.sc_offset = size - 3;
         goto more;
       case GST_H264_PARSER_BROKEN_DATA:
         GST_WARNING_OBJECT (h264parse, "input stream is corrupt; "
@@ -719,14 +771,24 @@ gst_h264_parse_check_valid_frame (GstBaseParse * parse,
         data, nalu.offset, nalu.size);
 
     gst_h264_parse_process_nal (h264parse, &nalu);
+
+    /* simulate no next nal if none needed */
+    drain = drain || (h264parse->align == GST_H264_PARSE_ALIGN_NAL);
+
+    /* In packetized mode we know there's only on NALU in each input packet,
+     * but we may not have seen the whole AU already, possibly need more */
+    if (h264parse->packetized_chunked) {
+      if (drain)
+        break;
+      /* next NALU expected at end of current data */
+      current_off = size;
+      goto more;
+    }
+
     /* if no next nal, we know it's complete here */
     if (drain || gst_h264_parse_collect_nal (h264parse, data, size, &nalu))
       break;
 
-    /* In packetized mode we know there's only on NALU in each input packet */
-    if (h264parse->packetized)
-      break;
-
     GST_DEBUG_OBJECT (h264parse, "Looking for more");
   }
 
@@ -1029,7 +1091,7 @@ gst_h264_parse_update_src_caps (GstH264Parse * h264parse, GstCaps * caps)
       /* but not necessarily or reliably this */
       if (h264parse->fps_num > 0 && h264parse->fps_den > 0)
         gst_caps_set_simple (caps, "framerate",
-            GST_TYPE_FRACTION, sps->fps_num, sps->fps_den, NULL);
+            GST_TYPE_FRACTION, h264parse->fps_num, h264parse->fps_den, NULL);
     }
   }
 
@@ -1209,9 +1271,12 @@ gst_h264_parse_parse_frame (GstBaseParse * parse, GstBaseParseFrame * frame)
 
   gst_h264_parse_update_src_caps (h264parse, NULL);
 
-  gst_h264_parse_get_timestamp (h264parse,
-      &GST_BUFFER_TIMESTAMP (buffer), &GST_BUFFER_DURATION (buffer),
-      h264parse->frame_start);
+  /* don't mess with timestamps if provided by upstream,
+   * particularly since our ts not that good they handle seeking etc */
+  if (h264parse->do_ts)
+    gst_h264_parse_get_timestamp (h264parse,
+        &GST_BUFFER_TIMESTAMP (buffer), &GST_BUFFER_DURATION (buffer),
+        h264parse->frame_start);
 
   if (h264parse->keyframe)
     GST_BUFFER_FLAG_UNSET (buffer, GST_BUFFER_FLAG_DELTA_UNIT);
@@ -1445,6 +1510,9 @@ gst_h264_parse_pre_push_frame (GstBaseParse * parse, GstBaseParseFrame * frame)
           /* collect result and push */
           new_buf = gst_byte_writer_reset_and_get_buffer (&bw);
           gst_buffer_copy_metadata (new_buf, buffer, GST_BUFFER_COPY_ALL);
+          /* should already be keyframe/IDR, but it may not have been,
+           * so mark it as such to avoid being discarded by picky decoder */
+          GST_BUFFER_FLAG_UNSET (new_buf, GST_BUFFER_FLAG_DELTA_UNIT);
           gst_buffer_replace (&frame->buffer, new_buf);
           gst_buffer_unref (new_buf);
         }
@@ -1572,8 +1640,20 @@ gst_h264_parse_set_caps (GstBaseParse * parse, GstCaps * caps)
     }
   }
 
-  /* negotiate with downstream, sets ->format and ->align */
-  gst_h264_parse_negotiate (h264parse);
+  {
+    GstCaps *in_caps;
+
+    /* prefer input type determined above */
+    in_caps = gst_caps_new_simple ("video/x-h264",
+        "parsed", G_TYPE_BOOLEAN, TRUE,
+        "stream-format", G_TYPE_STRING,
+        gst_h264_parse_get_string (h264parse, TRUE, format),
+        "alignment", G_TYPE_STRING,
+        gst_h264_parse_get_string (h264parse, FALSE, align), NULL);
+    /* negotiate with downstream, sets ->format and ->align */
+    gst_h264_parse_negotiate (h264parse, in_caps);
+    gst_caps_unref (in_caps);
+  }
 
   if (format == h264parse->format && align == h264parse->align) {
     gst_base_parse_set_passthrough (parse, TRUE);
@@ -1682,6 +1762,24 @@ gst_h264_parse_event (GstBaseParse * parse, GstEvent * event)
       gst_event_replace (&h264parse->force_key_unit_event, event);
       break;
     }
+    case GST_EVENT_FLUSH_STOP:
+      h264parse->dts = GST_CLOCK_TIME_NONE;
+      h264parse->ts_trn_nb = GST_CLOCK_TIME_NONE;
+      break;
+    case GST_EVENT_NEWSEGMENT:
+    {
+      gdouble rate, applied_rate;
+      GstFormat format;
+      gint64 start;
+
+      gst_event_parse_new_segment_full (event, NULL, &rate, &applied_rate,
+          &format, &start, NULL, NULL);
+      /* don't try to mess with more subtle cases (e.g. seek) */
+      if (format == GST_FORMAT_TIME &&
+          (start != 0 || rate != 1.0 || applied_rate != 1.0))
+        h264parse->do_ts = FALSE;
+      break;
+    }
     default:
       break;
   }
@@ -1763,7 +1861,17 @@ gst_h264_parse_chain (GstPad * pad, GstBuffer * buffer)
             nalu.data + nalu.offset, nalu.size);
         /* at least this should make sense */
         GST_BUFFER_TIMESTAMP (sub) = GST_BUFFER_TIMESTAMP (buffer);
-        GST_LOG_OBJECT (h264parse, "pushing NAL of size %d", nalu.size);
+        /* transfer flags (e.g. DISCONT) for first fragment */
+        if (nalu.offset <= nl)
+          gst_buffer_copy_metadata (sub, buffer, GST_BUFFER_COPY_FLAGS);
+        /* in reverse playback, baseparse gathers buffers, so we cannot
+         * guarantee a buffer to contain a single whole NALU */
+        h264parse->packetized_chunked =
+            (GST_BASE_PARSE (h264parse)->segment.rate > 0.0);
+        h264parse->packetized_last =
+            (nalu.offset + nalu.size + nl >= GST_BUFFER_SIZE (buffer));
+        GST_LOG_OBJECT (h264parse, "pushing NAL of size %d, last = %d",
+            nalu.size, h264parse->packetized_last);
         ret = h264parse->parse_chain (pad, sub);
       } else {
         /* pass-through: no looking for frames (and nal processing),
